import translate from "./translate-hints.js";

// Global state for overview
var current_scene = "";
var current_seed = Number.MAX_VALUE;
var current_checks = 0;
var current_entrances = 0;
var current_hints = 0;
var current_codes = Number.MAX_VALUE;
var total_checks = 0;
var total_entrances = 0;

// Global state for all entrances
// This should hold numbers for how many entrances are mapped/total per scene.
var current_entrances_list = {};

// Global state for all checks
// This should hold numbers for how many checks are cleared/total per scene.
var current_checks_list = {};

// Global state internal
var server_address = "";
var cross_codes = {};

window.onload = () => {
	get_updated_server_address();
	fetch(`${document.URL}static/tracker/data/holy_cross_codes.json`)
		.then((response) => response.json())
		.then(
			(data) => {
				cross_codes = JSON.parse(JSON.stringify(data));
				refresh_overview().then((overview) => {
					perform_updates(true, true, true, true);
					update_overview(overview, true, true);
				});
				refresh_elements();
			},
			(error) => {
				console.log(error);
			}
		);
};

async function refresh_elements() {
	const overview = await refresh_overview().catch(() =>
		console.log("Could not access the API server.")
	);

	if (overview) {
		document.getElementById("status-block").classList.add("hidden");

		const changed_seed = overview.seed != current_seed;
		const changed_scene = overview.scene != current_scene;
		const changed_checks = overview.items != current_checks;
		const changed_entrances = overview.entrances != current_entrances;
		const changed_codes = Object.keys(overview.codes).length != current_codes;
		const changed_hints = overview.hints != current_hints;
		const any_change =
			changed_seed ||
			changed_scene ||
			changed_checks ||
			changed_entrances ||
			changed_codes ||
			changed_hints;

		if (any_change) {
			await perform_updates(
				changed_seed,
				changed_checks,
				changed_entrances,
				changed_hints
			);
			update_overview(overview, changed_scene, changed_codes);
		}
	}
	setTimeout(refresh_elements, 500);
}

async function perform_updates(
	changed_seed,
	changed_checks,
	changed_entrances,
	changed_hints
) {
	if (changed_seed) {
		// TODO: this at some point *needs* to initialize all the data when the seed changes, instead of relying on the initial HTML.
		update_checks(await refresh_checks());
		update_entrances(await refresh_entrances());
		update_hints(await refresh_hints());
		console.log(`Seed changed to: ${current_seed}`);
	} else {
		if (changed_checks) {
			refresh_checks().then((data) => update_checks(data));
		}
		if (changed_entrances) {
			refresh_entrances().then((data) => update_entrances(data));
		}
		if (changed_hints) {
			refresh_hints().then((data) => update_hints(data));
		}
	}
}

function notices_ur_debug() {
	document.getElementById("debug-block").classList.toggle("hidden");
}

async function get_updated_server_address() {
	fetch(`${document.URL}get/address`)
		.then(
			(response) => response.json(),
			(error) => {
				console.log("Are you sure the front end is up?");
			}
		)
		.then(
			(data) => {
				const parsed_data = JSON.parse(JSON.stringify(data));
				server_address = parsed_data["listen_address"];
				if (parsed_data["backend_filepath_updated"]) {
					get_updated_filepath();
				}
			},
			(error) => {
				console.log(error);
			}
		);
}

async function refresh_overview() {
	const response = await fetch(`${server_address}overview`);
	const data = await response.json();
	return data;
}

async function refresh_hints() {
	const response = await fetch(`${server_address}hints`);
	const data = await response.json();
	return data;
}

async function refresh_checks() {
	const response = await fetch(`${server_address}items`);
	const data = await response.json();
	return data;
}

async function refresh_entrances() {
	const response = await fetch(`${server_address}doors`);
	const data = await response.json();
	return data;
}

async function update_overview(overview, changed_scene, changed_codes) {
	let overview_checks_title = document
		.getElementById("overview-totals")
		.querySelector(".overview-checks");
	let overview_entrances_title = document
		.getElementById("overview-totals")
		.querySelector(".overview-entrances");

	// Set content to updated data.
	overview_checks_title.textContent = `${overview.items}/${total_checks} (${
		total_checks - overview.items
	} left)`;
	overview_entrances_title.textContent = `${
		overview.entrances * 2
	}/${total_entrances} (${total_entrances - overview.entrances * 2} left)`;

	current_seed = overview.seed;
	current_scene = overview.scene;
	current_checks = overview.items;
	current_entrances = overview.entrances;
	current_hints = overview.hints;
	current_codes = Object.keys(overview.codes).length;

	if (changed_scene) {
		await update_scene(overview.scene);
	}
	if (changed_codes) {
		await update_codes(overview.codes);
	}
}

async function update_codes(codes) {
	// Codes that are always active
	const default_cross_codes = cross_codes.Default;

	// One time codes independent of scene
	const global_cross_codes = cross_codes.Global;

	let new_cross_codes_block_list = document
		.getElementById("codes-list")
		.cloneNode(true);
	let cross_codes_block_list_item = document
		.getElementById("codes-list")
		.firstElementChild.cloneNode(true);
	new_cross_codes_block_list.innerHTML = "";
	new_cross_codes_block_list.appendChild(
		cross_codes_block_list_item.cloneNode(true)
	);
	cross_codes_block_list_item.classList.remove("hidden");

	Object.keys(default_cross_codes).forEach((code) => {
		cross_codes_block_list_item.querySelector(
			".codes-list-item-title"
		).textContent = code;
		cross_codes_block_list_item.querySelector(
			".codes-list-item-code"
		).textContent = default_cross_codes[code]
			.replace(/U/g, "⬆️")
			.replace(/R/g, "➡️")
			.replace(/D/g, "⬇️")
			.replace(/L/g, "⬅️");
		new_cross_codes_block_list.appendChild(
			cross_codes_block_list_item.cloneNode(true)
		);
	});
	Object.keys(global_cross_codes).forEach((code) => {
		cross_codes_block_list_item.querySelector(
			".codes-list-item-title"
		).textContent = code;
		cross_codes_block_list_item.querySelector(
			".codes-list-item-code"
		).textContent = global_cross_codes[code]
			.replace(/U/g, "⬆️")
			.replace(/R/g, "➡️")
			.replace(/D/g, "⬇️")
			.replace(/L/g, "⬅️");
		new_cross_codes_block_list.appendChild(
			cross_codes_block_list_item.cloneNode(true)
		);
	});
	Object.keys(codes).forEach((code) => {
		cross_codes_block_list_item.querySelector(
			".codes-list-item-title"
		).textContent = code;
		cross_codes_block_list_item.querySelector(
			".codes-list-item-code"
		).textContent = cross_codes[current_scene][code]
			.replace(/U/g, "⬆️")
			.replace(/R/g, "➡️")
			.replace(/D/g, "⬇️")
			.replace(/L/g, "⬅️");
		new_cross_codes_block_list.appendChild(
			cross_codes_block_list_item.cloneNode(true)
		);
	});
	document.getElementById("codes-list").replaceWith(new_cross_codes_block_list);
}

async function apply_summary_colors(summary) {
	// Apply color coding to summary block
	summary.element.classList.remove(
		"from-highlight-both-light",
		"from-highlight-checks-light",
		"from-highlight-entrances-light",
		"from-highlight-empty-light",
		"from-highlight-undiscovered-light",
		"to-highlight-both-dark",
		"to-highlight-checks-dark",
		"to-highlight-entrances-dark",
		"to-highlight-empty-dark",
		"to-highlight-undiscovered-dark",
		"text-highlight-both-text",
		"text-highlight-checks-text",
		"text-highlight-entrances-text",
		"text-highlight-empty-text",
		"text-highlight-undiscovered-text"
	);

	if (summary.checks_total > 0 && summary.entrances_total > 0) {
		if (
			summary.checks_collected == summary.checks_total &&
			summary.entrances_found == summary.entrances_total
		) {
			summary.element.classList.add(
				"from-highlight-empty-light",
				"to-highlight-empty-dark",
				"text-highlight-empty-text"
			);
		} else if (
			summary.checks_collected <= 0 &&
			summary.entrances_found <= 0 &&
			summary.entrances_total > 0
		) {
			summary.element.classList.add(
				"from-highlight-undiscovered-light",
				"to-highlight-undiscovered-dark",
				"text-highlight-undiscovered-text"
			);
		} else if (summary.checks_collected >= summary.checks_total) {
			summary.element.classList.add(
				"from-highlight-checks-light",
				"to-highlight-checks-dark",
				"text-highlight-checks-text"
			);
		} else if (summary.entrances_found >= summary.entrances_total) {
			summary.element.classList.add(
				"from-highlight-entrances-light",
				"to-highlight-entrances-dark",
				"text-highlight-entrances-text"
			);
		} else {
			summary.element.classList.add(
				"from-highlight-both-light",
				"to-highlight-both-dark",
				"text-highlight-both-text"
			);
		}
	}
}

async function update_summary(updates) {
	if ("checks" in updates) {
		const summary_checks = document
			.getElementById("summary-list")
			.querySelector(`[data-scene="${updates.scene}"]`)
			.querySelector(".summary-checks");
		summary_checks.textContent = `Checks: ${updates.checks.collected}/${summary_checks.dataset.checksTotal} (${updates.checks.remaining} left)`;
		summary_checks.dataset.checksUndiscovered = updates.checks.collected;
		apply_summary_colors({
			element: document
				.getElementById("summary-list")
				.querySelector(`[data-scene="${updates.scene}"]`).firstElementChild,
			checks_collected: updates.checks.collected,
			checks_total: summary_checks.dataset.checksTotal,
			entrances_found: document
				.getElementById("summary-list")
				.querySelector(`[data-scene="${updates.scene}"]`)
				.querySelector(".summary-entrances").dataset.entrancesUndiscovered,
			entrances_total: document
				.getElementById("summary-list")
				.querySelector(`[data-scene="${updates.scene}"]`)
				.querySelector(".summary-entrances").dataset.entrancesTotal,
		});
	} else if ("entrances" in updates) {
		const summary_entrances = document
			.getElementById("summary-list")
			.querySelector(`[data-scene="${updates.scene}"]`)
			.querySelector(".summary-entrances");
		summary_entrances.textContent = `Entrances: ${updates.entrances.found}/${summary_entrances.dataset.entrancesTotal} (${updates.entrances.remaining})`;
		summary_entrances.dataset.entrancesUndiscovered = updates.entrances.found;
		apply_summary_colors({
			element: document
				.getElementById("summary-list")
				.querySelector(`[data-scene="${updates.scene}"]`).firstElementChild,
			checks_collected: document
				.getElementById("summary-list")
				.querySelector(`[data-scene="${updates.scene}"]`)
				.querySelector(".summary-checks").dataset.checksUndiscovered,
			checks_total: document
				.getElementById("summary-list")
				.querySelector(`[data-scene="${updates.scene}"]`)
				.querySelector(".summary-checks").dataset.checksTotal,
			entrances_found: updates.entrances.found,
			entrances_total: summary_entrances.dataset.entrancesTotal,
		});
	} else {
		console.log("What the heck is this");
	}
}

async function update_checks(checks) {
	total_checks = checks.total;

	let new_breakdown_block_checks_list = document
		.getElementById("breakdown-list")
		.firstElementChild.querySelector(".breakdown-block-checks-list")
		.cloneNode(true);
	let breakdown_block_checks_list_item = document
		.getElementById("breakdown-list")
		.firstElementChild.querySelector(".breakdown-block-checks-list")
		.firstElementChild.cloneNode(true);

	// Create new lists with updated data.
	let breakdown_list = Array.from(
		document.getElementById("breakdown-list").children
	);
	breakdown_list.forEach((scene) => {
		// Create variables for element pointers.
		let scene_title = scene.querySelector(".breakdown-block-title").textContent;
		if (scene_title) {
			const scene_checks = parseInt(
				scene.querySelector(".breakdown-block-checks-title").dataset.checks
			);
			if (scene_checks != checks.scenes[scene_title].collected) {
				scene.querySelector(".breakdown-block-checks-title").dataset.checks =
					scene_checks;
				update_summary({
					scene: scene_title,
					checks: {
						collected: checks.scenes[scene_title].collected,
						remaining: checks.scenes[scene_title].remaining,
					},
				});
				scene.querySelector(
					".breakdown-block-checks-title"
				).textContent = `Checks: ${checks.scenes[scene_title].collected}/${checks.scenes[scene_title].total} (${checks.scenes[scene_title].remaining})`;

				new_breakdown_block_checks_list.innerHTML = "";
				breakdown_block_checks_list_item.classList.remove("hidden");
				Object.keys(checks.scenes[scene_title].checks).forEach((check) => {
					if (checks.scenes[scene_title].checks[check].name == "") {
						breakdown_block_checks_list_item.textContent = `❌ ${check}`;
						new_breakdown_block_checks_list.appendChild(
							breakdown_block_checks_list_item.cloneNode(true)
						);
					}
				});
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene_title}"]`)
					.querySelector(".breakdown-block-checks-list").innerHTML = "";
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene_title}"]`)
					.querySelector(".breakdown-block-checks-list")
					.replaceWith(new_breakdown_block_checks_list.cloneNode(true));
			}
		}
	});
	// current_checks = checks.collected;
}

async function update_entrances(entrances) {
	total_entrances = entrances.total;

	let new_breakdown_block_entrances_list = document
		.getElementById("breakdown-list")
		.firstElementChild.querySelector(".breakdown-block-entrances-list")
		.cloneNode(true);
	let breakdown_block_entrances_list_item = document
		.getElementById("breakdown-list")
		.firstElementChild.querySelector(".breakdown-block-entrances-list")
		.firstElementChild.cloneNode(true);
	let new_breakdown_block_mapped_list = document
		.getElementById("breakdown-list")
		.firstElementChild.querySelector(".breakdown-block-mapped-list")
		.cloneNode(true);
	let breakdown_block_mapped_list_item = document
		.getElementById("breakdown-list")
		.firstElementChild.querySelector(".breakdown-block-mapped-list")
		.firstElementChild.cloneNode(true);
	// Create new lists with updated data.
	let breakdown_list = Array.from(
		document.getElementById("breakdown-list").children
	);
	breakdown_list.forEach((scene) => {
		// Create variables for element pointers.
		let scene_title = scene.querySelector(".breakdown-block-title").textContent;
		if (scene_title) {
			const scene_entrances = parseInt(
				scene.querySelector(".breakdown-block-entrances-title").dataset
					.entrances
			);
			if (scene_entrances != entrances.scenes[scene_title].found) {
				scene.querySelector(".breakdown-block-checks-title").dataset.entrances =
					scene_entrances;
				update_summary({
					scene: scene_title,
					entrances: {
						found: entrances.scenes[scene_title].found,
						remaining: entrance.scenes[scene_title].remaining,
					},
				});
				scene.querySelector(
					".breakdown-block-entrances-title"
				).textContent = `Entrances: ${entrances.scenes[scene_title].found}/${entrances.scenes[scene_title].total} (${entrances.scenes[scene_title].remaining})`;

				new_breakdown_block_entrances_list.innerHTML = "";
				breakdown_block_entrances_list_item.classList.remove("hidden");
				new_breakdown_block_mapped_list.innerHTML = "";
				breakdown_block_mapped_list_item.classList.remove("hidden");

				Object.keys(entrances.scenes[scene_title].doors).forEach((entrance) => {
					if (entrances.scenes[scene_title].doors[entrance].door == "") {
						breakdown_block_entrances_list_item.textContent = `❌ ${entrance}`;
						new_breakdown_block_entrances_list.appendChild(
							breakdown_block_entrances_list_item.cloneNode(true)
						);
					} else {
						breakdown_block_mapped_list_item.textContent = `✔️ ${entrance} -> ${entrances.scenes[scene_title].doors[entrance].door}`;
						breakdown_block_mapped_list_item.id = `${entrance}-mapped`;
						breakdown_block_mapped_list_item.dataset.scene =
							entrances.scenes[scene_title].doors[entrance].scene;
						new_breakdown_block_mapped_list.appendChild(
							breakdown_block_mapped_list_item.cloneNode(true)
						);
					}
				});
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene_title}"]`)
					.querySelector(".breakdown-block-entrances-list").innerHTML = "";
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene_title}"]`)
					.querySelector(".breakdown-block-entrances-list")
					.replaceWith(new_breakdown_block_entrances_list.cloneNode(true));
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene_title}"]`)
					.querySelector(".breakdown-block-mapped-list").innerHTML = "";
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene_title}"]`)
					.querySelector(".breakdown-block-mapped-list")
					.replaceWith(new_breakdown_block_mapped_list.cloneNode(true));
			}
		}
	});
}

async function update_hints(hints) {
	let hints_list = document.getElementById("hints-list").cloneNode(true);
	let hints_list_item = hints_list.firstElementChild.cloneNode(true);
	hints_list.innerHTML = "";
	Object.keys(hints).forEach((hint_index) => {
		let hint = hints[hint_index].split(
			/(\[[\w\s]+?\]|\"[\w \d\>\<#.\-\']+\"|\<[\w\d#]+\>)/gm
		);
		hint = hint.map((segment) => {
			segment = segment.trim();
			if (segment) {
				if (
					!(
						segment.startsWith("[") ||
						segment.startsWith("<") ||
						segment.startsWith('"')
					)
				) {
					segment = translate(segment.trim());
				} else if (segment.startsWith("[")) {
					segment = segment.trim();
				} else {
					segment = ` ${segment.trim()} `;
				}
				return segment;
			}
		});
		hints[hint_index] = hint.join("");
	});
	Object.keys(hints).forEach((hint_index) => {
		let hint = hints[hint_index];
		let matches = Array.from(
			hint.matchAll(/\<([\w\d#]+)\>(.*)(\<[\w\d#]+\>)/gm)
		);
		if (matches.length >= 1) {
			hint = hint.replace(
				/(\<[\w\d#]+\>.*\<[\w\d#]+\>)/gm,
				`<span style="color: ${matches[0][1]};">${matches[0][2]}</span>`
			);
		}
		hint = hint.replace(/ "|" /gm, " ");
		hints_list_item.firstElementChild.innerHTML = hint;
		hints_list.appendChild(hints_list_item.cloneNode(true));
	});
	document.getElementById("hints-list").innerHTML = "";
	document.getElementById("hints-list").replaceWith(hints_list);
}

async function update_scene(scene) {
	Array.from(document.getElementById("breakdown-list").children).forEach(
		(breakdown) => {
			const breakdown_title = breakdown.querySelector(
				".breakdown-block-title"
			).textContent;
			if (breakdown_title == scene) {
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene}"]`)
					.classList.remove("hidden");
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene}"]`).dataset.current =
					"true";
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene}"]`)
					.classList.remove("order-last");
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene}"]`)
					.classList.add("order-first");
			} else {
				document
					.getElementById("breakdown-list")
					.querySelector(
						`[data-breakdown-scene="${breakdown_title}"]`
					).dataset.current = "false";

				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${breakdown_title}"]`)
					.classList.add("hidden");
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${scene}"]`)
					.classList.add("order-last");
				document
					.getElementById("breakdown-list")
					.querySelector(`[data-breakdown-scene="${breakdown_title}"]`)
					.classList.remove("order-first");
			}
		}
	);
	Array.from(
		document.getElementById("summary-list").firstElementChild.children
	).forEach((summary) => {
		const summary_scene = summary.dataset.scene;
		if (summary_scene == scene) {
			document
				.getElementById("summary-list")
				.firstElementChild.querySelector(`[data-scene="${scene}"]`)
				.classList.add("hidden");
		} else if (!(summary_scene == "")) {
			document
				.getElementById("summary-list")
				.firstElementChild.querySelector(`[data-scene="${summary_scene}"]`)
				.classList.remove("hidden");
		}
	});
}
